@@ -159,14 +159,16 @@ WriteMemoryCallback(void* ptr, size_t size, size_t nmemb, void* data)
 static char* get_captcha(guint32 captcha_sid)
 {
     g_debug("get_captcha START");
-    char* url;
+    gchar* url;
     char* img;
 
     url = g_strdup_printf("http://userapi.com/data?act=captcha&csid=%d", captcha_sid);
 
-    if (http_get(url, info_proxy, info_port, NULL, &img) == 200)
+    if (http_get(url, info_proxy, info_port, NULL, &img) == 200){
+        g_free(url);
         return img;
-
+    }
+    g_free(url);
     g_debug("get_captcha END");
     return NULL;
 }
@@ -184,11 +186,11 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     int captch = 0;
     int init_status = 0;
     char* url = NULL;
-    char* remixmid = NULL;
-    char* remixsid = NULL;
-    char* class = NULL;
-    char* function = NULL;
-    char* tmp = NULL;
+    gchar* remixmid = NULL;
+    gchar* remixsid = NULL;
+    gchar* class = NULL;
+    gchar* function = NULL;
+    gchar* tmp = NULL;
     xmlXPathObject *Obj = NULL;
 
     global_temp++;
@@ -206,6 +208,8 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     class = get_req_class(reqXml);
     function = get_req_function(reqXml);
     xmlSetProp(rootNode, CLASS, class);
+    g_free(class);
+    class = NULL;
     xmlSetProp(rootNode, FUNCTION, function);
     
     node = xmlNewDocNode(doc, NULL,   PARAMS,  NULL);
@@ -215,6 +219,7 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     g_debug("function = %s", function);
     if (g_strcmp0(function, GET_SETTINGS) == 0) {
         get_settings(doc);
+        g_free(function);
        return 0;
     }
     
@@ -224,9 +229,11 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
         } else {
             info_message("410", doc, DRV_SETT_SAVED, 0);
         }
+       g_free(function);
        return 0;
     }
     if (g_strcmp0(function, CAPTCHA_MESSAGE) == 0) {
+        g_free(function);
         captch = 1;
 
         char* captcha_code = NULL;
@@ -243,15 +250,12 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
 
         Obj = xpath("//Request/Params/string[@name='oldRequest']/Request", reqXml);
 	
-	if(Obj != NULL) {
-		g_debug("OBG != NULL");
+	if(Obj != NULL){
             node = Obj->nodesetval->nodeTab[0];
-	    g_debug("GET NODE");
             childNode = xmlCopyNode(node,1);
-	    g_debug("SET");
+            xmlUnlinkNode(childNode);
             reqXml2 = xmlNewDoc("1.0");
             xmlDocSetRootElement(reqXml2, childNode);
-	    g_debug("SET");
             xmlXPathFreeObject(Obj);
         } else {
             return 1;
@@ -259,13 +263,16 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     }
 
     if(captch) {
-	    class = get_req_class(reqXml2);
-    	function = get_req_function(reqXml2);
-	} 
-	
-    xmlSetProp(rootNode, CLASS, class);
-    xmlSetProp(rootNode, FUNCTION, function);
-
+        class = get_req_class(reqXml2);
+        function = get_req_function(reqXml2);
+        xmlSetProp(rootNode, CLASS, class);
+        xmlSetProp(rootNode, FUNCTION, function);
+    }
+    if(class)
+        g_free(class);
+    if(function)
+        g_free(function);
+  
     init_status = initialization(&remixmid, &remixsid);
     attempts = 0;
     if (init_status != 0) {
@@ -278,11 +285,11 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
                 return 0;
         }
         if (init_status == 2){
-            info_message("403", doc, DRV_NETW_ERROR, 1);
+            info_message(DRV_NETW_ERROR_CODE, doc, DRV_NETW_ERROR, 1);
             return 0;
         }
         if(init_status == 1) {
-            info_message("402", doc, DRV_AUTH_ERROR, 1);
+            info_message(DRV_AUTH_ERROR_CODE, doc, DRV_AUTH_ERROR, 1);
             return 0;
         }
                  
@@ -314,8 +321,8 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
     gchar* converted_friends = NULL;
     char* tmp = NULL;
     char* url;
-    char* class = get_req_class(reqXml);
-    char* function = get_req_function(reqXml);
+    //gchar* class = get_req_class(reqXml);
+    gchar* function = get_req_function(reqXml);
     int num = 0;
     int init_status = 0;
 
@@ -326,14 +333,19 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
     if (url == NULL || strlen(url)  < 1 ) {
         if (g_strcmp0(function, GET_BINARY_DATA) == 0 ) {
             counter = -1;
+            g_free(function);
             return 0;
         } else {
-            info_message("402", doc, DRV_REQ_ERROR, 1);
+            info_message(DRV_REQ_ERROR_CODE, doc, DRV_REQ_ERROR, 1);
+            g_free(function);
             return 0;
         }
     }
-    if (http_get(url, info_proxy, info_port, NULL, &tmp) == 200) {
+    int code = 0;
+    code = http_get(url, info_proxy, info_port, NULL, &tmp);
+    if (code == 200) {
         if (g_strcmp0(function, GET_BINARY_DATA) == 0 ) {
+            g_free(function);
             if (counter == -1) {
                 return 0;
             }
@@ -348,22 +360,23 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
         switch (check(tmp)) {
             case 0:
                 if (g_strcmp0(function, SEND_MESSAGE) == 0) {
-                    info_message("405", doc, DRV_MESS_ERROR, 1);
+                    info_message(DRV_MESS_ERROR_CODE, doc, DRV_MESS_ERROR, 1);
 		    g_free(tmp);
+                    g_free(function);
                     return 0;
                 }
-                g_debug("check OK");
             case 1:
 
                 if (g_strcmp0(function, GET_MY_PROFILE) == 0 || g_strcmp0(function, UPDATE_PROFILE) == 0) {
                     if(parser_profile(doc, tmp, remixmid) == 1){
-                        info_message("403", doc, DRV_RESP_ERROR, 1);
+                        info_message(DRV_RESP_ERROR_CODE, doc, DRV_RESP_ERROR, 1);
                     }
                     g_free(tmp);      	    
                 }
                 if (g_strcmp0(function, SEND_MY_PROFILE) == 0) {
-                    info_message("400", doc, DRV_PROF_SAVED, 0);
+                    info_message(DRV_PROF_SAVED_CODE, doc, DRV_PROF_SAVED, 0);
                     g_free(tmp);
+                    g_free(function);
                     return 0;
                 }
                 if (g_strcmp0(function, GET_LIST_FRIENDS) == 0) {
@@ -375,6 +388,7 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                         counter++;
                         make_request( reqXml, doc, remixmid, remixsid);
                     } else {
+                        g_free(function);
                         return 0;
                     }
                 }
@@ -385,6 +399,7 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                         counter++;
                         make_request( reqXml, doc, remixmid, remixsid);
                     } else {
+                        g_free(function);
                         return 0;
                     }
                 }
@@ -401,17 +416,19 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                             g_free(tmp);
 			    read_message(doc, remixmid, remixsid);
                         } else {
-                            info_message("403", doc, DRV_NETW_ERROR, 1);
+                            info_message(DRV_NETW_ERROR_CODE, doc, DRV_NETW_ERROR, 1);
                         }
 
                     }
                 }
 
                 if (g_strcmp0(function, SEND_MESSAGE) == 0) {
-                    info_message("400", doc, DRV_MESS_SEND, 0);
+                    info_message(DRV_MESS_SEND_CODE, doc, DRV_MESS_SEND, 0);
                     g_free(tmp);
+                    g_free(function);
                     return 0;
                 }
+                g_free(function);
                 break;
             case -1:
                 unset_settings(REMIXSID);
@@ -422,22 +439,30 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                         make_request(reqXml, doc, remixmid, remixsid);
                 }
                 else 		
-                    info_message("401", doc, DRV_AUTH_ERROR, 1);
+                    info_message(DRV_AUTH_ERROR_CODE, doc, DRV_AUTH_ERROR, 1);
+                g_free(function);
                 g_free(tmp);
 		break;
             case -2:
                 captcha_message(doc, reqXml);
                 g_free(tmp);
+                g_free(function);
 		break;
             case -3:
                 g_debug(tmp);
-                info_message("403", doc, DRV_RESP_ERROR, 1);
+                info_message(DRV_ACCESS_ERROR_CODE, doc, DRV_ACCESS_ERROR, 1);
+                g_free(function);
                 g_free(tmp);
 		break;
         }
 
-    } else {
-        info_message("403", doc, DRV_NETW_ERROR, 1);
+    } else if (code >= 500){
+        info_message(DRV_SERVER_ERROR_CODE, doc, DRV_SERVER_ERROR, 1);
+        g_free(function);
+        return 0;
+    } else if (code == 400){
+        info_message(DRV_NETW_ERROR_CODE, doc, DRV_NETW_ERROR, 1);
+        g_free(function);
         return 0;
     }
     return 0;
@@ -588,6 +613,7 @@ int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     int status = 0;
     gchar* path;
     gchar* temp;
+    gchar* name;
     xmlNodePtr rootNode, childNode, node;
 
     client = gconf_client_get_default ();
@@ -597,21 +623,26 @@ int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
 
     while(rootNode != NULL) {
         g_debug("%s", rootNode->name);
-        if (g_strcmp0(xmlGetProp(rootNode, NAME),LOGIN) == 0) {
+        name = xmlGetProp(rootNode, NAME);
+        if (g_strcmp0(name, LOGIN) == 0) {
             path = g_strconcat(PATH, LOGIN, NULL);
             temp = xmlNodeGetContent(rootNode);	    
             checked = gconf_client_set_string (client, path, temp, NULL);
             g_free(path);    
 	    g_free(temp);
         }
-        if (g_strcmp0(xmlGetProp(rootNode, NAME),PASSWORD) == 0) {
+        g_free(name);
+        name = xmlGetProp(rootNode, NAME);
+        if (g_strcmp0(name, PASSWORD) == 0) {
             path = g_strconcat(PATH, PASSWORD, NULL);
 	    temp = xmlNodeGetContent(rootNode);
             checked = gconf_client_set_string (client, path, temp, NULL);
             g_free(path);
 	    g_free(temp);
         }
-        if (g_strcmp0(xmlGetProp(rootNode, NAME),AUTOUPDATE) == 0) {
+        g_free(name);
+        name = xmlGetProp(rootNode, NAME);
+        if (g_strcmp0(name, AUTOUPDATE) == 0) {
             int time = 0;
 	    temp = (char*)xmlNodeGetContent(rootNode);
             time = atoi(temp);
@@ -623,7 +654,9 @@ int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
             g_free(path);
 	    g_free(temp);
         }
-        if (g_strcmp0(xmlGetProp(rootNode, NAME),STATUS) == 0) {
+        g_free(name);
+        name = xmlGetProp(rootNode, NAME);
+        if (g_strcmp0(name, STATUS) == 0) {
             path = g_strconcat(PATH, STATUS, NULL);
 	    temp = xmlNodeGetContent(rootNode);
 #ifndef TEST
@@ -633,8 +666,8 @@ int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
             g_free(path);
 	    g_free(temp);
         }
+        g_free(name);
         rootNode = rootNode->next;
-
         if(!checked)
             status = 1;
     }
@@ -681,7 +714,7 @@ int authorize(char *email, char *pass)
     char* mid = NULL, sid[MID_SIZE];
     char *tmp;
     char *resp;
-    char* post_data;
+    gchar* post_data;
     int code = 0;
     struct curl_slist *cookies;
     gchar* path;
@@ -697,12 +730,12 @@ int authorize(char *email, char *pass)
     post_data = g_strdup_printf("email=%s&pass=%s&expire=0&vk=", email, pass);
 
 #ifdef TEST
-    info_proxy = g_strdup("proxy.karelia.ru");
-    info_port = 81;
+    info_proxy = CTF_TEST_DATA("PROXY");
+    info_port = atoi(CTF_TEST_DATA("PORT"));
 #endif
 
     code = http_post("http://login.vk.com/?act=login", info_proxy, info_port, post_data, NULL, &body, &cookies);
-
+    g_free(post_data);
     if (code == 200) {
 
         int i = 0;
@@ -859,9 +892,12 @@ int authorize(char *email, char *pass)
  */
 char* get_req_class(xmlDocPtr reqXml)
 {
-    xmlNodePtr rootNode;
-    rootNode = xmlDocGetRootElement(reqXml);
-   return xmlGetProp(rootNode, CLASS);
+    xmlNodePtr rootNode = NULL;
+    if(reqXml != NULL)
+        rootNode = xmlDocGetRootElement(reqXml);
+    if (rootNode != NULL)
+        return xmlGetProp(rootNode, CLASS);
+    return NULL;
 }
 
 /**
