@@ -583,7 +583,7 @@ int parser_messages(char *json, xmlDocPtr doc)
         node2 = xmlAddChild(node, node2);
         g_strfreev(name_list);
 
-        g_debug("parse_time");
+        //g_debug("parse_time");
         time = json_object_array_get_idx(message,1);
         if(time == NULL)
             break;
@@ -596,7 +596,7 @@ int parser_messages(char *json, xmlDocPtr doc)
         node2 = xmlAddChild(node, node2);
         g_free(tmp_str);
 
-        g_debug("parse text");
+        //g_debug("parse text");
         text = json_object_array_get_idx(message,2);
         if(text == NULL)
             break;
@@ -623,7 +623,7 @@ int parser_messages(char *json, xmlDocPtr doc)
         node2 = xmlAddChild(node, node2);
         g_free(temp);
         
-        g_debug("parse read_status");
+        //g_debug("parse read_status");
         status = json_object_array_get_idx(message,5);
         node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, json_object_get_string(status));
         xmlSetProp(node2, NAME, "Status");
