@@ -26,7 +26,13 @@
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
-
+const char* response_string = "\
+<TransitData>\
+<TargetID></TargetID>\
+<SourceID>vkontakte</SourceID>\
+<Content>\
+</Content>\
+</TransitData>";
 /**
 * Функция отправки данных драйвером
 */
@@ -63,6 +69,7 @@ extern int msa_module_init(msa_module* d)
     int conf = 0;
 
 g_debug("before gconf");
+    g_type_init();
     client = gconf_client_get_default ();
 g_debug("after gconf");
 
@@ -118,101 +125,91 @@ g_debug("after gconf");
  */
 static int vk_driver_send(xmlDocPtr request, xmlDocPtr* response, msa_module* info)
 {
-    xmlDocPtr reqXml = NULL, bufXml = NULL;
-    xmlDocPtr doc = NULL;
+    xmlDocPtr reqXml = NULL, respXml = NULL;
+    xmlDocPtr buffXml = NULL;
     if (info == NULL) {
-        g_debug("default proxy settings");
-        info_proxy = g_strdup("proxy.karelia.ru");
-        info_port = 81;
+        g_debug("msa_module in NULL! can't perform request!");
+        return FAILURE;
     } else {
         g_debug("get proxy settings %s %d ", info->proxy, info->port);
         info_proxy = info->proxy;
-        info_port = info->port;    
+        info_port = info->port;
     }
 
     g_debug("vk_driver_send: START");
-    //xmlDocDump(stdout, request);
 
-    doc = xmlNewDoc(BAD_CAST "1.0");
-    doc->encoding = g_strdup(UTF8);
+    respXml = xmlNewDoc(BAD_CAST "1.0");
+    respXml->encoding = g_strdup(UTF8);
 
 
-	if(request == NULL)
-	{
-		printf("Error! Can't open request.xml\n");
-		return 1;
-	}
+    if(request == NULL) {
+        printf("Error! Can't read request\n");
+        return FAILURE;
+    }
 
-    xmlNodePtr node = NULL, node2 = NULL;
-    node = xmlDocGetRootElement(request);
-    node = node->children;
-    char *source;
+    xmlXPathObject *Obj = NULL;
+    xmlNodePtr node = NULL, node_request = NULL, node_response = NULL;
+    gchar* source;
     
-    while (node != NULL) {
-        if (strcmp(node->name, "Content") == 0) {
-            node = node->children;
-            //xmlUnlinkNode(node);
-            //node2 = xmlCopyNode(node, 1); 
-
-            bufXml = xmlNewDoc(BAD_CAST "1.0");
-            bufXml->encoding = UTF8;
-            xmlDocSetRootElement(bufXml, node);
-            break;
-        } else if(strcmp(node->name, "SourceID") == 0) {
-                source = xmlNodeGetContent(node);
-                }
-        node = node->next;
-    }
-
-    int i = 0;
-    //xmlDocDump(stdout, bufXml);
-    vk_req(bufXml, doc, info);
-    //xmlFreeDoc(bufXml);    
-
-    //xmlDocDump(stdout,doc);
-
-    int *size;
-    char* tmp;
-    xmlChar* temp2;
-
-    node = xmlDocGetRootElement(doc);
-    node2 = xmlCopyNode(node, 1);
-    //xmlFreeNode(node);
-    xmlFreeDoc(doc);
-    reqXml = xmlNewDoc( BAD_CAST g_strdup("1.0"));
-    reqXml->encoding = g_strdup(UTF8);
-
-    node = xmlNewDocNode(reqXml, NULL, "TransitData", NULL);
-    xmlDocSetRootElement(reqXml, node);
-
-    xmlNodePtr child;
-    child = xmlNewDocNode(reqXml, NULL, "SourceID", NULL);
-    xmlNodeSetContent(child, "vkontakte");
-    xmlAddChild(node, child);
-
-    child = xmlNewDocNode(reqXml, NULL, "TargetID", NULL);
-    xmlNodeSetContent(child, source);
-    xmlAddChild(node, child);
-    g_free(source);
-
-    child = xmlNewDocNode(reqXml, NULL, "Content", NULL);
-    xmlAddChild(node, child);
-
-    node = node->children;
-
-    while (node != NULL) {
-        if (strcmp(node->name, "Content") ==  0) {
-            xmlAddChild(node, node2);
-            break;
-        }
-        node = node->next;  
-    }
-    //xmlDocDump(stdout, reqXml);
-    *response = xmlCopyDoc(reqXml, 1);
+    Obj = xpath("//TransitData/SourceID", request);
+    if(Obj != NULL) {
+        node = Obj->nodesetval->nodeTab[0];
+        source = xmlNodeGetContent(node);
+        xmlXPathFreeObject(Obj);
+    } else {
+        return FAILURE;
+      }
+    
+    Obj = xpath("//TransitData/Content/Request", request);
+    if(Obj != NULL) {
+        node = Obj->nodesetval->nodeTab[0];
+        node_request = xmlCopyNode(node, 1);
+        xmlUnlinkNode(node_request);
+        reqXml = xmlNewDoc(BAD_CAST "1.0");
+        reqXml->encoding = g_strdup(UTF8);
+        xmlDocSetRootElement(reqXml, node_request);
+        xmlXPathFreeObject(Obj);
+    } else {
+        return FAILURE;
+      }
     xmlFreeDoc(request);
+    
+    vk_req(reqXml, respXml, info);
     xmlFreeDoc(reqXml);
-    //g_debug("!!!!!!!");
-    return 0;
+
+    node = xmlDocGetRootElement(respXml);
+    node_response = xmlCopyNode(node, 1);
+    xmlUnlinkNode(node_response);
+    xmlFreeDoc(respXml);
+    
+    //buffXml = xmlNewDoc( BAD_CAST "1.0");
+
+    buffXml = xmlParseDoc(BAD_CAST response_string);
+    buffXml->encoding = g_strdup(UTF8);
+    
+    Obj = xpath("//TransitData/TargetID", buffXml);
+    if(Obj != NULL) {
+        node = Obj->nodesetval->nodeTab[0];
+        xmlNodeSetContent(node, source);
+        xmlXPathFreeObject(Obj);
+    } else {
+        return FAILURE;
+    }
+
+    Obj = xpath("//TransitData/Content", buffXml);
+    if(Obj != NULL){
+        node = Obj->nodesetval->nodeTab[0];
+        xmlAddChild(node, node_response);
+        xmlXPathFreeObject(Obj);
+    } else {
+        return FAILURE;
+    }
+    
+    *response = xmlCopyDoc(buffXml, 1);
+    xmlFreeDoc(buffXml);
+    g_free(source);
+    g_debug("vk_driver_send: END");
+    return SUCCESS;
 }
 
 /**
